#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Datos: Todas las clases relacionadas con los datos, y sus correspondientes metodos

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import variables
import types

class Registro(list):
	"""Clase que guarda un registro"""
	def __init__(self,variables):
		""" 
		Clase constructora
		variables es el array de variables al que esta asociado el registro
		Añade los valores por defecto para todos los atributos
		"""
		self.variables=variables
		list.__init__(self)
		[self.append(x)	for x in self.variables.arr_defaults()]

	def __getitem__(self,indice):
		""" Redefinicion del [] de la lista. 
		Si nos pasan una ristra, comprobamos si se corresponde a alguna variable
		Si nos pasan un objeto, comprobamos si se corresponde a alguna variable
		Si nos pasan un entero, devolvemos el valor de esa posicion
		"""
		miindice=self.variables.obtenerindice(indice)
		return list.__getitem__(self,miindice)

	def __setitem__(self,indice,valor):
		""" 
			Redefinicion de []= de la lista
			Almacena en el valor convertido por la variable asociada
		"""
		miindice=self.variables.obtenerindice(indice)
		list.__setitem__(self,miindice,self.variables[miindice].interno(valor))

	def append(self,valor):
		""" Redefinicion del append de lista """
		tama=self.__len__()
		intermedio=self.variables[tama].interno(valor)
		list.append(self,intermedio)
		
	def set(self,lista):
		"""Esta funcion establece los valores del registro"""
		i=0
		for valor in lista:
			self.__setitem__(i,valor)
			i+=1


class Listaficheros(list):
	"""Contenedor de la lista de ficheros"""
	def __init__(self,n):
		self.nelementos=n
	def insert(self,elemento):
		if elemento in self:
			self.remove(elemento)
		list.insert(self,0,elemento)
		if len(self)>self.nelementos:
			self.pop()


class Arrayvar(list): 
	"""Lista que guarda las variables. Hereda de list"""
	def __init__(self):
		"""Función de inicialización"""
		list.__init__(self)

	def obtenerindice(self,parametro):
		"""
		Obtiene el indice de un determinado tipo, 
		-Por la cadena del nombre
		-por la referencia al objeto en si 
		-o por el indice entero
		"""
		if parametro.__class__==types.StringType:
			for iter in self:  
				if iter.nombre == parametro:
					return self.index(iter)
			return False
		elif parametro.__class__==types.IntType:
			return parametro
		else: #El parametro lo tomamos como una referencia
			if parametro in self:
				return self.index(parametro)
			return False

	def arr_defaults(self):
		"""Devuelve un array con todos los valores por defecto"""
		return [x.valorpordefecto for x in self]


class contenedorregvar:
	"""Contenedor de registros y variables"""
	def __init__(self):
		"""Inicialización del contenedor principal"""
		self.datos=[] #Un array para datos
		self.vars=Arrayvar() #Un array de variables 

	def __getitem__(self,indice): #Acceso por []
		"""Devuelve el item segun un indice numerico, un nombre, o un objeto variable
			Si es un indice, se entiende que se quiere acceder a un registro
			Si es un nombre o el objeto variable, se entiende que se quiere acceder a la columna
		"""
		if indice.__class__==types.IntType:
			return self.datos[indice]
		else:
			return self.__get_col(indice)
			
	def __setitem__(self,indice,valor): 
		"""
		Modificacion por []
		Si lo que se pasa es una string o un objeto, se entiende que se quiere
		asignar de golpe una columna
		"""
		if indice.__class__==types.IntType:
			self.datos[indice]=valor
		else: 
			for registro in range(len(self.datos)):
				self.datos[registro][indice]=valor[registro]

	def __delitem__(self,indice):
		"""Redefinicion del borrado, borra un registro"""
		if indice.__class__==types.IntType:
			self.datos.__delitem__(indice)

	def get_dat(self,indice=-1):
		"""Devuelve un registro, por defecto el último"""
		return self.datos[indice]

	def get_var(self,indice=-1):
		"""Devuelve una variable, por defecto la última"""
		return self.vars[self.vars.obtenerindice(indice)] 

	def set_var(self,indice,valor):
		"""Escribe la variable asociada al indice"""
		self.vars[indice]=valor

	def num_dat(self):
		return len(self.datos)

	def num_var(self):
		return len(self.vars)

	def app_dat(self,dato=None):
		"""Añade un dato a los ya existentes"""
		if isinstance(dato,Registro):
			self.datos.append(dato)
		else:
			registro=Registro(self.vars)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.datos.append(registro)

	def ins_dat(self,pos,dato):
		"""inserta un dato en la posicion pos"""
		if isinstance(dato,Registro):
			self.datos.insert(pos,dato)
		else:
			registro=Registro(self.vars)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.datos.insert(pos,registro)
		

	def app_var(self,nombre=None,tipo="Entero",valores={},valorpordefecto=0,descripcion="",expresion=None):  
		"""Añade una variable de tipo tipo al array de variables. 
		Rellena los datos con el valor por defecto, salvo que se indique una expresion
		"""
		if not nombre:
			nombre="VAR"+str(self.num_var())
		if tipo=="Entero":
			self.vars.append(variables.Entero(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Booleano":
			self.vars.append(variables.Booleano(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Real":
			self.vars.append(variables.Real(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Fecha":
			self.vars.append(variables.Fecha(nombre,valores,valorpordefecto,descripcion)) 
		else:
			raise TypeError #PENDIENTE: Averiguar el tipo de excepción adecuado
		if expresion:
			for var in self.vars:
				numero=self.vars.obtenerindice(var)
				expresion=expresion.replace(var.nombre,"self.datos[i]["+repr(numero)+"]")
			for i in range(self.num_dat()):
				self.datos[i].append(eval(expresion)) 
		else: #No nos pasan expresion, rellenamos con los valores por defecto
			for i in range(self.num_dat()):
				self.datos[i].append(valorpordefecto)

	def arr_tit(self,tipo=None):
		"""
		Devuelve una lista con todos los nombres de las variables
		opcionalmente podemos filtrar segun un tipo
		"""
		if not tipo:
			return [variable.nombre for variable in self.vars]
		else:
			lista=[]
			for variable in self.vars:
				if cmp(variable.tipo,tipo)==0:
					lista.append(variable.nombre)
			return lista
				
	def arr_var(self):
		"""Devuelve el array con todos los objetos de las variables"""
		return [var for var in self.vars]
	
	def del_var(self,pos):
		"""Borra la variable de la posicion indicada """
		pass #PENDIENTE: Implementar

	def get_casos(self,indice):
		"""Devuelve todos los casos existentes de una determinada variable"""
		lista=[]
		for valor in self.datos:
			if valor[self.vars.obtenerindice(indice)] not in lista:
				lista.append(valor[self.vars.obtenerindice(indice)])
		lista.sort()
		return lista

	#Métodos privados de contenedorregvar

	def __get_col(self,indice):
		"""Devuelve un array con todos los valores asociados a la variable representada por indice 
		   La lista quedará ordenada
		"""
		lista=[]
		for valor in self.datos:
			lista.append(valor[self.vars.obtenerindice(indice)])
		return lista
			

