#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Fichero de interfaces
#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
import salida
from rpy import *  


#Clase base de interfaz
class Interfaz:
	"""Clase genérica de la que heredan los interfaces """
	def __init__(self,config,datos):
		self.config=config
		self.datos=datos

class InterfazR(Interfaz):
	"""Interfaz con R. Transforma una peticion de calculo en un objeto de salida"""

	def __init__(self,config,datos):
		"""Inicialización del interfaz R"""
		Interfaz.__init__(self,config,datos)

	def exportar(self,accion,variables,opciones):
		"""Exportacion de interfaz de datos/salida"""
		if accion=="descriptivo":
			#TODO Obtener todos los casos posibles, guardar en un array
			lvariables = [ x.latin1() for x in variables ] # Sin Qstring
			resultado=salida.descriptivo(opciones["agruparpor"],opciones["media"],opciones["varianza"],opciones["desviacion"],opciones["mediana"])
			media=None
			varianza=None
			desviacion=None
			mediana=None
			agruparpor=opciones["agruparpor"]
			if agruparpor:
				#Hay variable de agrupacion
				posiblescasos=self.datos.get_casos(agruparpor)
				for caso in posiblescasos:
					i=0
					for variable in lvariables:
						lista=[]
						for registro in self.datos.datos:
							if registro[agruparpor]==caso:
								lista.append(registro[lvariables[i]])
						if opciones["media"]:
							media=r.mean(lista)
						if opciones["varianza"]:
							varianza=r.var(lista)
						if opciones["desviacion"]:
							desviacion=r.sd(lista)
						if opciones["mediana"]:
							mediana=r.median(lista)
						resultado.insertarregistro(lvariables[i],caso,media,varianza,desviacion,mediana)
						i+=1


			else:
				#No hay variable de agrupacion
				i=0
				for variable in lvariables:
					lista=[int(x) for x in self.datos[lvariables[i]]]
					if opciones["media"]:
						media=r.mean(lista)
					if opciones["varianza"]:
						varianza=r.var(lista)
					if opciones["desviacion"]:
						desviacion=r.sd(lista)
					if opciones["mediana"]:
						mediana=r.median(lista)
					resultado.insertarregistro(lvariables[i],"",media,varianza,desviacion,mediana)
					i+=1
			return resultado 
		elif accion=="jicuadrado" or accion=="ks":
			if accion=="jicuadrado":
				resultado=salida.jicuadrado()
			else:
				resultado=salida.ks()
			lvariables = [ x.latin1() for x in variables ] # Sin Qstring
			num=self.datos.num_dat()
			i=0
			for variable in lvariables:
				lista=[int(x) for x in self.datos[lvariables[i]]]
				if opciones["distribucion"]=="uniforme":
					distribucion=r.runif(num,1./num,1./num)
					if accion=="jicuadrado":
						resultadoprueba=r.chisq_test(lista,p=distribucion)
					else:
						resultadoprueba=r.ks_test(lista,distribucion)
					resultado.insertarregistro(lvariables[i],opciones["percentil"],resultadoprueba)
				i+=1
			return resultado

		elif accion=="regresion":
			resultado=salida.regresion()
			lvariables1 = [ x[0].latin1() for x in variables ] # Sin Qstring
			lvariables2 = [ x[1].latin1() for x in variables ] # Sin Qstring
			num=self.datos.num_dat()
			for i in range(len(lvariables1)):
				lista1=[int(x) for x in self.datos[lvariables1[i]]]
				lista2=[int(x) for x in self.datos[lvariables2[i]]]
				resultadoprueba=r.lm(r("y ~ x"),data=r.data_frame(x=lista1, y=lista2))
				sumario=r.summary_lm(resultadoprueba,True)
				resultado.insertarregistro(lvariables1[i],lvariables2[i],resultadoprueba["coefficients"]["x"],resultadoprueba["coefficients"]["(Intercept)"],sumario["correlation"])
				#FIXME: La correlacion devuelve una matriz, no un valor, averiguar que representa cada valor
			return resultado

		elif accion=="barras":
			resultado=salida.barras()
			i=0
			lvariables = [ x.latin1() for x in variables ] # Sin Qstring
			for variable in lvariables:
				lista=[int(x) for x in self.datos[lvariables[i]]]
				r.png("/tmp/driza%03d.png") #Directorio temporal de la config
				r.hist(lista,main=lvariables[i],nclass=int(opciones["nclass"]))
				r.dev_off()
				#Enviar la ruta de la imagen y que la clase de salida lo convierta en qimage o lo que sea
				resultado.insertarregistro(lvariables[i])
				i+=1
			return resultado

		else:
			print "NO IMPLEMENTADO"
			return False

