#!/usr/bin/python
# -*- coding: utf-8 -*-
#####
# Fichero de variables
# Guarda todos los Variables relacionados con los Variables que maneja el programa
####

#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


class Variable:
	"""Variable generica. De esta clase heredan las Variables usados en el programa """
	def __init__(self,nombre,valores,valorpordefecto,descripcion):
		self.nombre=nombre
		self.valorpordefecto=valorpordefecto
		self.descripcion=descripcion
		if valores == None: self.valores={}
		else: self.valores=valores

	def update(self,objeto):
		"""Funcin de copia de los atributos para cualquier variable
		Sirve para hacer copias entre variables
		"""
		self.nombre=objeto.nombre
		self.valorpordefecto=objeto.valorpordefecto
		self.descripcion=objeto.descripcion
		if objeto.valores:
			self.valores=objeto.valores.copy()

	def __setitem__(self,indice,valor):
		"""Definicion del operador[]"""
		if indice==0:
			self.nombre=valor
		elif indice==3:
			self.valorpordefecto=valor
		elif indice==4:
			self.descripcion=valor

	def __getitem__(self,indice):
		"""Definicion del operador []="""
		if indice==0:
			return self.nombre
		elif indice==3:
			return self.valorpordefecto
		elif indice==4:
			return self.descripcion

class Entero(Variable):
	"""La clase entero. El formato interno ser "int" """
	def __init__(self,nombre=None,valores=None,valorpordefecto=0,descripcion=None):
		"""Inicializacion de variable entera"""
		Variable.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Entero"

	def interno(self,valor):
		"""transforma el valor al formato en el que se desea guardar, 
		comparando si un valor dado se ajusta al tipo de variable
		Genera una excepcin en caso de que la conversin no funcione
		"""
		return int(valor)

	def ristra(self,valor):
		"""Esta funcin transforma un valor en formato interno al formato de representacin (Cadena)"""
		return repr(valor)

class Real(Variable):
	"""La clase Real"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=None):
		"""Inicializacion del objeto Real"""
		Variable.__init__(self,nombre,valores,valorpordefecto)
		self.tipo="Real"

class Booleano(Variable):
	"""La clase Booleano, solo puede almacenar 1  0"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=None,descripcion=None):
		"""Inicializacin del objeto booleano"""
		Variable.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Booleano"
	def interno(self,valor):
		"""Paso al formato interno, si el dato no es vlido generar una excepcin"""
		if valor:
			return True
		else:
			return False
	def ristra(self,valor):
		"""Devuelve una ristra que representa al valor"""
		if valor:
			return "1"
		else:
			return "0"

class Fecha(Variable):
	"""La clase Fecha define el tipo que representa una fecha"""
	def __init__(self,nombre,valores,valorpordefecto):
		"""Inicializacion de fecha"""
		Variable.__init__(self,nombre,valores,valorpordefecto)

